#! /usr/bin/env python
# -*- coding: utf-8 -*-
################################################################################
# Copyright (c) 2016, Perceptive Automation, LLC. All rights reserved.
# SEPCIAL THANK YOU TO EVERYONE WHO HAS EVER WRITTEN A PLUGIN FOR INDIGO
# AS THIS WAS MY MAIN PLACE OF UNDERSTANDING THE STRUCTURE!
################################################################################


################################################################################
# Imports
################################################################################
import indigo
import socket
import logging
import requests
import simplejson as json
################################################################################
# Globals
################################################################################
returnDeviceList = list()

_SERVER = 'https://home.sensibo.com/api/v2'
_SLEEP = 15
_SENSIBOSERVERDOWN = False


################################################################################
class Plugin(indigo.PluginBase):
    ########################################
    # Class properties
    ########################################
    def logmessage(self, message, logtype, isPluginError):
       
 
        if self.logLevel =="1":
            indigo.server.log(unicode(message),  type=self.pluginDisplayName + ' Debug', isError=False)

        if isPluginError == True:
            indigo.server.log(unicode(message),  type=self.pluginDisplayName + ' Error', isError=True)
                
    
    
    ########################################
    def __init__(self, pluginId, pluginDisplayName, pluginVersion, pluginPrefs):
        super(Plugin, self).__init__(pluginId, pluginDisplayName, pluginVersion, pluginPrefs)
        self.logLevel = pluginPrefs.get("pluginloglevel", "0")

        self.apikey = pluginPrefs.get('apikey', None)
        self.stopThread = False
        self._runSensibo = True

        self.logmessage(u'Entered: Func init',1,False)
        self.logmessage(u'Sensibo API Key is %s' % (self.apikey),1, False )
        self.logmessage(u'Plugin Debug Log is %s' % (self.logLevel),1, False )
    
    ########################################
    def startup(self):
        self.logmessage(u'Entered: Func startup',1,False)
        self.updateAllSensiboLists()
 
    ########################################
    def updateAllSensiboLists(self):
        self.logmessage(u'Entered: Func updateAllSensiboLists',1,False)
        #this function will do something
        if self.apikey is None:
            self.logmessage(u'No API Key Set - Unable to conitnue',1,False)
            #self.log("No API has been set - please use the plugin config page to set the API Key",0)
            errorText = u"No Api Key Set"
            self.errorLog(errorText)
            self.lastErrorMessage=errorText
            return

        #continue to get the senibo device list
        self.getSeniboDevices()

    #######################################
    def getSeniboDevices(self):
        self.logmessage(u'Entered: Func getSeniboDevices',1,False)

        try:
            
            sensibodevicelist = self.devices()

            for room in sensibodevicelist:
                self.logmessage(u'Found Device '+ room,1,False)
                returnDeviceList.append([sensibodevicelist[room], room])
    

        except requests.exceptions.Timeout:
            errorText = u"Failed to retreive data from sensibo - check settings and retry."
            # Don't display the error if it's been displayed already.
            if errorText != self.lastErrorMessage:
                self.errorLog(errorText)
                self.logmessage(u'Request timeout while talking to sensibo',1,True)
                # Remember the error.
                self.lastErrorMessage = errorText
                
        except requests.exceptions.ConnectionError:
            errorText = u"Failed to connect to home.senibo.com - Check Sensibo is working"
            # Don't display the error if it's been displayed already.
            if errorText != self.lastErrorMessage:
                self.errorLog(errorText)
                self.logmessage(u'Connection error while talking to sensibo',1,true)
                # Remember the error.
                self.lastErrorMessage = errorText
                return

        except Exception, e:
            self.logmessage(u'Unable to obtain data from Sensibo ' + unicode(e),1,True)

    ########################################
    def shutdown(self):
        self.logmessage(u'Entered: Func shutdown - Requested to shutdown BYE!',1,False)
    
    ########################################
    def runConcurrentThread(self):
        self.logmessage(u'Entered: Func runConcurrentThread',1,False)
        self.logmessage(u'Starting Sensibo Update',0,False)
        try:
            while self._runSensibo:
                self.updatesensibo()
                self.sleep(_SLEEP)
        except self.StopThread:
            self.debugLog("stopping sensibo")

        except Exception, e:
             self.logmessage(u'Plugin Stopping.  Reason = %s' % (e) ,0,True)

    def updatesensibo(self):
        self.logmessage(u'Entered: Func update sensibo',1,False)
        prId = "com.duncangrant.indigoplugin.sensibo"

        if _SENSIBOSERVERDOWN == False:
            for sensibo in indigo.devices.iter(prId):
                    ############################################################################################
                    # Get The current sensibo device unitid from the list of devices belonging to this plugin  #
                    # this value was stored as a device property during the device config validation method.   #
                    # SEE closedDeviceConfigUi for details.                                                    #
                    ############################################################################################
                    
                thisdeviceprops = sensibo.pluginProps
                
                sensibounitid = thisdeviceprops['devicetype']
                self.logmessage(u'Found sensibo unit id :' + sensibounitid,1,False)

                mesresult = self.pod_measurement(sensibounitid)
                self.logmessage(u'Data Received ' + unicode(mesresult),1,False)
                
                for mes in mesresult:

                    jsonstring = json.dumps(mes)
                    sensibojson = json.loads(jsonstring)
                    self.logmessage(u'Updating Device States Temperature %s and Humidity %s ' %(float(sensibojson["temperature"]),float(sensibojson["humidity"])) ,1,False)
              
                    sensibo.updateStateOnServer("humidity",float(sensibojson["humidity"]),decimalPlaces=1)
                    sensibo.updateStateOnServer("temperature",float(sensibojson["temperature"]),decimalPlaces=1)
                
                
                acresult = self.pod_ac_state(sensibounitid)
                self.logmessage(u'AC State Returned :' + unicode(acresult),1,False)
               
                
                sensibo.updateStateOnServer("fanLevel",acresult["fanLevel"])
                sensibo.updateStateOnServer("power",bool(acresult["on"]))
                sensibo.updateStateOnServer("mode",acresult["mode"])
                sensibo.updateStateOnServer("temperatureUnit",acresult["temperatureUnit"])
                sensibo.updateStateOnServer("targetTemperature",acresult["targetTemperature"])
                sensibo.updateStateOnServer("online",True)
        else:
  
            self.logmessage(u'Sensibo service is not online.  Will retry later',0,False)
            for sensibo in indigo.devices.iter(prId):
                 #Sensibo Server is offline so we cannot do much about it.
                 #mark all devicea as offline until the server is back online.
                 sensibo.updateStateOnServer("online",False)


    ################################################################################
    # METHOD IS CALLED FROM INDIGO AFTER THE USER CLICKS SAVE ON THE DEVICE CONFIG #
    ################################################################################
    def validateDeviceConfigUi(self, valuesDict, typeId, devId):
        self.logmessage(u'Entered: Func validateDeviceConfigUI ' + unicode(valuesDict),1,False)
        return (True, valuesDict)
    
    #### RETURN THE LIST OF DEVICE FOUND FOR THIS USERS API
    def deviceListGenerator(self, filter="", valuesDict=None, typeId="", targetId=0):
        self.logmessage(u'Entered: Func deviceListGenerator',1,False)
        return returnDeviceList

    def _get(self, path, ** params):
        self.logmessage(u'Entered: Func _get',1,False)
        params['apiKey'] = self.apikey
        try:
            # See if the newValue is a number.
            response = requests.get(_SERVER + path, params = params)
            response.raise_for_status()
            self.debugLog(response.json())
            _SENSIBOSERVERDOWN = False
            return response.json()
        except ValueError:
            # This isn't a number, don't try to make it one.
            _SENSIBOSERVERDOWN = True
            self.logmessage(u'Sensibo API is not responding at this time',0,True)
            pass
        
      
    
    def _patch(self, path, data, ** params):
        params['apiKey'] = self.apikey
        response = requests.patch(_SERVER + path, params = params, data = data)
        response.raise_for_status()
        return response.json()
    
    def devices(self):
        result = self._get("/users/me/pods", fields="id,room")
        return {x['room']['name']: x['id'] for x in result['result']}
    
    def pod_measurement(self, podUid):
        result = self._get("/pods/%s/measurements" % podUid)
        return result['result']
    
    def pod_ac_state(self, podUid):
        result = self._get("/pods/%s/acStates" % podUid, limit = 1, fields="status,reason,acState")
        return result['result'][0]['acState']


    def _post(self, path, data, ** params):
        params['apiKey'] = self.apikey
        response = requests.post(_SERVER + path, params = params, data = data)
        response.raise_for_status()
        return response.json()
        
    def pod_change_ac_state(self, podUid, powerstate, target_temperature, mode, fan_level):
        self._post("/pods/%s/acStates" % podUid,
                       json.dumps({'acState': {"on": powerstate, "targetTemperature": target_temperature, "mode": mode, "fanLevel": fan_level}}))

    # Closed Device Configuration
    ########################################
    def closedDeviceConfigUi(self, valuesDict, userCancelled, typeId, deviceId):
        self.logmessage(u'Entered: Func closeDeviceConfigUi :' + unicode(valuesDict),1,False)

        # If the user didn't cancel the changes, take any needed actions as a result of the changes made.
        if not userCancelled:
            # Configuration was saved.  Rebuild the device if needed.
            device = indigo.devices[deviceId]
            thisdeviceprops = device.pluginProps
            thisdeviceprops['unitId'] = valuesDict['devicetype']
            device.replacePluginPropsOnServer(thisdeviceprops)
            self.debugLog("Setting the unitId in device prperties " + valuesDict['devicetype']  )
            self.debugLog("Recalling this unitId to see if it has stored " + thisdeviceprops['unitId'] )
            self.debugLog("Recalling this unitId to see if it has stored " + device.pluginProps['unitId'] )

    # Validate Device Configuration
    ########################################
    def validateDeviceConfigUi(self, valuesDict, typeId, deviceId):
        self.logmessage(u'Entered: Func validateDeviceConfigUi',1,False)
        errorsDict = indigo.Dict()
        errorsDict['showAlertText'] = ""
        isError = False


        return (True, valuesDict)

    def validateActionConfigUi(self, valuesDict, typeId, deviceId):
        self.logmessage(u'Entered: Func validateActionConfigUi',1,False)
        errorsDict = indigo.Dict()
        errorsDict['showAlertText'] = ""
        isError = False
        device = indigo.devices[deviceId]
        
        upower = device.states['power']
        if valuesDict.get('unitpower', "") == True:
            if valuesDict.get('mode', "") == "":
                isError = True
                errorsDict['mode'] = u"Please select a mode."
                errorsDict['showAlertText'] += errorsDict['mode']
                return (False, valuesDict, errorsDict)
            if valuesDict.get('mode', "") == "":
                isError = True
                errorsDict['fanlevel'] = u"Please select a fan level."
                errorsDict['showAlertText'] += errorsDict['fanlevel']
                return (False, valuesDict, errorsDict)
            if valuesDict.get('mode', "") == "":
                isError = True
                errorsDict['targettemperature'] = u"Please select the Target Temperate."
                errorsDict['showAlertText'] += errorsDict['targettemperature']
                return (False, valuesDict, errorsDict)
    
        return (True, valuesDict)

    def closedPrefsConfigUi(self, valuesDict, UserCancelled):
        indigo.server.log("Config CLOSED " + unicode(valuesDict),  type=self.pluginDisplayName + ' Info', isError=False)
 
        self.logLevel = valuesDict['pluginloglevel']
        self.apikey = valuesDict['apikey']
        indigo.server.log("Logging Now %s" % self.logLevel ,  type=self.pluginDisplayName + ' Info', isError=False)
        return (True, valuesDict)

##########################################################################################################################
# ACTION CODE TO HANDLE THE ACTIONS.XML FILE
##########################################################################################################################

# Set Power
########################################
    def setPower(self, action, device):
        self.logmessage(u'Entered: Func setPower',1,False)
        upower = action.props.get('unitpower', False)

        thisdeviceprops = device.pluginProps
        sensibounitid = thisdeviceprops['devicetype']
        if _SENSIBOSERVERDOWN==False:
            if upower == "False":
                mode = device.states['mode']
                fanlevel = device.states['fanLevel']
                targetTemperature = device.states['targetTemperature']
                self.pod_change_ac_state(sensibounitid,False, int(targetTemperature), "%s" % mode, "%s" % fanlevel)
                device.updateStateOnServer("power",False)
            if upower == "True":
                #let's get the additional parameters needed to setup the AC unit
                mode = action.props.get('mode', False)
                fanlevel = action.props.get('fanlevel', 'low')
                targetTemperature = action.props.get('targettemperature', 20)
                self.pod_change_ac_state(sensibounitid,True, int(targetTemperature), "%s" % mode, "%s" % fanlevel)
                #Lets update the local device states so that any triggers don't get fired acceidently due to
                #time between refresh states
                device.updateStateOnServer("fanLevel",fanlevel)
                device.updateStateOnServer("power",True)
                device.updateStateOnServer("mode",mode)
                device.updateStateOnServer("targetTemperature",int(targetTemperature))
        else:
            self.logger.info("Cannot perform action right now.  Sensibo Servers are offline")

    #Request To Toggle the current power state of the device.
    def togglePower(self, action, device):
        self.logmessage(u'Entered: Func togglePower',1,False)
        thisdeviceprops = device.pluginProps
        sensibounitid = thisdeviceprops['devicetype']
        currentpowerstate = device.states['power']

        if currentpowerstate == True:
            mode = device.states['mode']
            fanlevel = device.states['fanLevel']
            targetTemperature = device.states['targetTemperature']
            self.pod_change_ac_state(sensibounitid,False, int(targetTemperature), "%s" % mode, "%s" % fanlevel)
            device.updateStateOnServer("power",False)
        else:
            mode = device.states['mode']
            fanlevel = device.states['fanLevel']
            targetTemperature = device.states['targetTemperature']
            self.pod_change_ac_state(sensibounitid,True, int(targetTemperature), "%s" % mode, "%s" % fanlevel)
            device.updateStateOnServer("power",True)


